@@ -88,7 +88,7 @@ void double_variable_queue_update_bitset(
         const std::vector<double> value,
         Rcpp::XPtr<individual_index_t> index
 ) {
-    if (index->max_size() != variable->size) {
+    if (index->max_size() != variable->size()) {
         Rcpp::stop("incompatible size bitset used to queue update for DoubleVariable");
     }
     auto index_vec = bitset_to_vector_internal(*index, false);
@@ -96,6 +96,26 @@ void double_variable_queue_update_bitset(
 }
 
 //[[Rcpp::export]]
-void double_variable_update(Rcpp::XPtr<DoubleVariable> variable) {
-    variable->update();
+void double_variable_queue_extend(
+    Rcpp::XPtr<DoubleVariable> variable,
+    std::vector<double>& values
+    ) {
+    variable->queue_extend(values);
+}
+
+//[[Rcpp::export]]
+void double_variable_queue_shrink(
+    Rcpp::XPtr<DoubleVariable> variable,
+    std::vector<size_t>& index
+    ) {
+    decrement(index);
+    variable->queue_shrink(index);
+}
+
+//[[Rcpp::export]]
+void double_variable_queue_shrink_bitset(
+    Rcpp::XPtr<DoubleVariable> variable,
+    Rcpp::XPtr<individual_index_t> index
+    ) {
+    variable->queue_shrink(*index);
 }
